import greenfoot.*;
import java.awt.Color;

/**
 * This is the underlying class Creature, the center lifeform of this barren universe.
 * No, seriously, it is. reatures can eat, die, fight, and reproduce. The reatures colors 
 * are determined by their DNA.
 * 
 * @author TriMill
 * @version 1.0.1
 */
public class Creature extends SmoothMover
{
    /**
     * The internal clock of the Creature
     */
    private int time = Greenfoot.getRandomNumber(20)+1;
    /**
     * The Creature's DNA
     */
    private CreatureDNA dna;
    /**
     * How full the Creature's belly is.
     */
    private double food;
    /**
     * How long it lives
     */
    private static final int LIFESPAN = 18001;
    /**
     * How long ago it bred.
     */
    public int breedTimer = 0;
    
    /**
     * Creates a new random creature.
     */
    public Creature()
    {
        dna = new CreatureDNA(10);
        food = 5.5;
        construct();
    }
    
    /**
     * Creates a creature from an egg
     */
    public Creature(Egg egg) 
    {
        dna = egg.getDNA();
        food = 2;
        construct();
    }
    
    /**
     * This method contains the things that are common to both constructors.
     * The coloring of the creature depends upon its DNA. Speed determines redness, 
     * rotation when it hits food determines greenness, and random rotations 
     * determine blueness.
     */
    private void construct()
    {
        GreenfootImage img = getImage();
        img.scale(dna.getSize()*10, dna.getSize()*11);
        img.rotate(90);
        turn(dna.getInitialTurn());
        int red = (int)(((dna.getSpeed() - 0.8)/4.2) * 255.0);
        int green = (int)((Math.abs(dna.getFoodTurn())/180.0) * 255.0);
        double ave = Math.abs((dna.getEvenTurn() + dna.getOddTurn())/2.0);
        int blue = (int)(((ave / 90.0))*255);
        if(red>255){red=255;} if(red<0){red=0;}
        if(green>255){green=255;} if(green<0){green=0;}
        if(blue>255){blue=255;} if(blue<0){blue=0;}
        colorize(new Color(red, green, blue));
    }
        
    /**
     * Move, Eat, Turn, Reproduce, get Hungry, and Die. All in one method. MIND = BLOWN.
     * 
     * First, the creature moves ann amount dictated by its DNA. Then, it turns if the internal 
     * clock time is divisible by 100. If it is dividible by 200 turn one amount, if it isnt, turn another 
     * amount. Then, if the creature is an an edge it will be wrapped around to the other side. Then, if it
     * is old enough, it will try to reproduce. It will only be succesful if there is another creature
     * touching it. Then, it gets hungry, at a rate dependent upon how fast it goes. Finally, it checks if it
     * should die, decrements it's breed timer, and then increments its internal clock. 
     * 
     * There, only 8 lines!
     */
    public void act() 
    {
        move(dna.getSpeed());
        
        tryToEat();
        if((time % 100) == 0)
        {
            if((time % 200) == 0)
                turn(dna.getEvenTurn());
            else
                turn(dna.getOddTurn());
        }
        checkEdges();
        if(time > 555)
            tryProduceOffspring();
        
        if(time % (4701/(dna.getSpeed() + dna.getSize()*30)) == 0)
            food--;
        if(food <= 0 || time > LIFESPAN)
        {
            DeadCreature body = new DeadCreature();
            try 
            {
                getWorld().addObject(body, getX(), getY());
            }
            catch(Exception ex) { System.out.println("Unable to die."); }
            getWorld().removeObject(this);
        }
        
        if(breedTimer > 0) 
            breedTimer--;
        
        time++;
    }    
    
    /**
     * Tries to eat things it comes ito contact with. Only eats when hungry
     */
    public void tryToEat() 
    {
        if(food <= 13)
        {
            if(isTouching(Food.class))
            {
                for(Object o: getIntersectingObjects(Food.class))
                    food = food + 0.9;
                getWorld().removeObjects(getIntersectingObjects(Food.class));
                turn(dna.getFoodTurn());
            }
            
            if(isTouching(SuperFood.class))
            {
                for(Object o: getIntersectingObjects(SuperFood.class))
                    food = food + 2.6;
                getWorld().removeObjects(getIntersectingObjects(SuperFood.class));
                turn(dna.getFoodTurn());
            }
            
            if(isTouching(DeadCreature.class))
            {
                for(Object o: getIntersectingObjects(DeadCreature.class))
                    food = food + 2.025;
                getWorld().removeObjects(getIntersectingObjects(DeadCreature.class));
                turn(dna.getFoodTurn());
            }
            
            if(isTouching(Egg.class) && food < 3.75 && breedTimer < 75)
            {   
                food = food + 0.275;
                getWorld().removeObject(getOneIntersectingObject(Egg.class));
                turn(dna.getFoodTurn());
            }
        }
    }
    
    /**
     * If on edges, wrap around to the other side of the world.
     * Model for a spherial world.
     */
    public void checkEdges() 
    {
        if(onXEdge())
            setLocation(getWorld().getWidth()-getX(), getY());
        if(onYEdge())
            setLocation(getX(), getWorld().getHeight()-getY());
    }
    
    /**
     * Don't worry, it's simpler than in real life.
     * 
     * First, it must be full enough, touching another creature, and not have recently bred 
     * (to prevent "Death Breeding".) Then, the creatures have an five percent chence of fighting. If they 
     * fight, the one who is faster, bigger,  and fuller will win. If they breed, a random number of eggs 
     * are produced, each bearing DNA. The DNA are created by choosing random genomes from the parents.
     * Finally, both parents loose some food and their breeding timers set back up, the mates timer a bit 
     * longer than the original.
     * 
     * Mabey it is more complicated than real life.
     */
    public void tryProduceOffspring()
    {
        // Simpler than in real life
        
        if(isTouching(Creature.class) && breedTimer <= 0 && food > 5)
        {
            // Five percent chance of a fight
            if(Greenfoot.getRandomNumber(100) < 5) 
            {
                Creature opponent = (Creature)getOneIntersectingObject(Creature.class);
                double myFitness = dna.getFitness(food);
                double opponentFitness = opponent.dna.getFitness(opponent.food);
                if(myFitness > opponentFitness)
                {
                    // I eat them
                    food = food + 3;
                    getWorld().removeObject(opponent);
                }
                else if(myFitness < opponentFitness)
                {
                    // They eat me
                    opponent.food = opponent.food + 3;
                    getWorld().removeObject(this);
                }
            }
            else
            {
                Creature mate = (Creature)getOneIntersectingObject(Creature.class);
                if(mate.breedTimer > 0) return;
                int eggs = Greenfoot.getRandomNumber(3) + 1;
                for(int i = 0; i < eggs; i++)
                {
                    // Please don't look, we like our privacy
                    int xoff = Greenfoot.getRandomNumber(11) - 5;
                    int yoff = Greenfoot.getRandomNumber(11) - 5;
                    Egg egg = new Egg(this, mate);
                    getWorld().addObject(egg, mate.getX() + xoff, mate.getY() + yoff);
                }
                food = food - (food/7.5);
                mate.food = mate.food - (mate.food/7.5);
                mate.breedTimer = 201;
                this.breedTimer = 200;
            }
        }
    }
    
    /**
     * Makes the Creature have color. For loop designed by Mik.It is the best For loop ever.
     */
    private void colorize(Color newCol)
    {
        
        GreenfootImage img = getImage();
        int red = 0;
        int green = 0;
        int blue = 0;
        
        // Thanks, Mik, for this awesome bit of code!!!
        for (int x = 0; x < img.getWidth(); x++) {
            for (int y = 0; y < img.getHeight(); y++) {
                Color pixel = img.getColorAt(x, y);
                int alpha = pixel.getAlpha();
                if (alpha != 0) {
                    double intensity = pixel.getGreen() / 128.0;  // mid value == 1, less is darker, more is brighter
                    red = (int)(newCol.getRed() * intensity);
                    if (red > 255) red = 255;
                    green = (int)(newCol.getGreen() * intensity);
                    if (green > 255) green = 255;
                    blue = (int)(newCol.getBlue() * intensity);
                    if (blue > 255) blue = 255;
                }
                img.setColorAt (x, y, new Color(red, green, blue, alpha));
            }
        }
    }
    
    /** 
     * Gets the creature's DNA
     * 
     * @return The Creature's DNA
     */
    public CreatureDNA getDNA()
    {
        return dna;
    }
    
    /**
     * Gets the creature's food level
     * 
     * @return The Creature's food level
     */
    public double getFood()
    {
        return food;
    }
}
