import greenfoot.*;

/**
 * An egg produced by a creature. After a certian amount of time, it
 * will hatch.
 * 
 * @author TriMill 
 * @version 1.0.1
 */
public class Egg extends Actor
{
    /**
     * The DNA of the creture in the egg.
     */
    private CreatureDNA dna;
    /**
     * The time it takes for the egg to hatch.
     */
    private static final int HATCHTIME = 3500;
    /**
     * The internal clock of the egg.
     */
    private int time = 0;
    
    /**
     * Creates a new egg from two parents
     */
    public Egg(Creature parent1, Creature parent2)
    {
        dna = new CreatureDNA(parent1.getDNA(), parent2.getDNA());
    }
    
    /**
     * Checks to see if the egg should hatch. If it should, there is
     * a 65 percent chance the baby lives.
     */
    public void act() 
    {
        if(time > HATCHTIME)
        {
            // 65% chance the egg survives
            if(Greenfoot.getRandomNumber(100) < 65)
            {
                Creature baby = new Creature(this);
                getWorld().addObject(baby, getX(), getY());
            }
            getWorld().removeObject(this);
        }
        // Keep the time tickin'
        time++;
    }    
    
    /**
     * Returns the DNA of the egg
     * 
     * @return The DNA of this egg
     */
    public CreatureDNA getDNA() 
    {
        return dna;
    }
}
